<?php
/**
 * ###############################################
 *
 * SWIFT Framework
 * _______________________________________________
 *
 * @author		Varun Shoor
 *
 * @package		SWIFT
 * @copyright	Copyright (c) 2001-2012, Kayako
 * @license		http://www.kayako.com/license
 * @link		http://www.kayako.com
 *
 * ###############################################
 */

/**
 * The Loop Rule View Management Class
 *
 * @author Varun Shoor
 */
class View_LoopRule extends SWIFT_View
{
	/**
	 * Constructor
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function __construct()
	{
		parent::__construct();

		return true;
	}

	/**
	 * Destructor
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function __destruct()
	{
		parent::__destruct();

		return true;
	}

	/**
	 * Render the Parser Loop Rule Form
	 *
	 * @author Varun Shoor
	 * @param int $_mode The Render Mode
	 * @param object $_SWIFT_LoopRuleObject The SWIFT_LoopRule Object Pointer (Only for EDIT Mode)
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function Render($_mode, SWIFT_LoopRule $_SWIFT_LoopRuleObject = null)
	{
		$_SWIFT = SWIFT::GetInstance();

		if (!$this->GetIsClassLoaded())
		{
			return false;
		}

		// Calculate the URL
		if ($_mode == SWIFT_UserInterface::MODE_EDIT)
		{
			$this->UserInterface->Start(get_class($this), '/Parser/LoopRule/EditSubmit/' . $_SWIFT_LoopRuleObject->GetLoopRuleID(),
					SWIFT_UserInterface::MODE_EDIT, true);
		} else {
			$this->UserInterface->Start(get_class($this), '/Parser/LoopRule/InsertSubmit', SWIFT_UserInterface::MODE_INSERT, false);
		}

		$_loopRuleTitle = '';
		$_loopRuleLength = 600;
		$_loopRuleMaxHits = 2;
		$_loopRuleRestoreAfter = 600;

		if ($_mode == SWIFT_UserInterface::MODE_EDIT)
		{
			if ((isset($_POST['_isDialog']) && $_POST['_isDialog'] == 1) || $this->UserInterface->IsAjax() == false)
			{
				$this->UserInterface->Toolbar->AddButton($this->Language->Get('update'), 'icon_check.gif');
			}

			$this->UserInterface->Toolbar->AddButton($this->Language->Get('delete'), 'icon_delete.gif', '/Parser/LoopRule/Delete/' .
					$_SWIFT_LoopRuleObject->GetLoopRuleID(), SWIFT_UserInterfaceToolbar::LINK_CONFIRM, '', '', false);
			$this->UserInterface->Toolbar->AddButton($this->Language->Get('help'), 'icon_help.gif', SWIFT_Help::RetrieveHelpLink('parserlooprule'),
					SWIFT_UserInterfaceToolbar::LINK_NEWWINDOW);

			$_loopRuleTitle = $_SWIFT_LoopRuleObject->GetProperty('title');
			$_loopRuleLength = intval($_SWIFT_LoopRuleObject->GetProperty('length'));
			$_loopRuleMaxHits = intval($_SWIFT_LoopRuleObject->GetProperty('maxhits'));
			$_loopRuleRestoreAfter = intval($_SWIFT_LoopRuleObject->GetProperty('restoreafter'));
		} else {
			$this->UserInterface->Toolbar->AddButton($this->Language->Get('insert'), 'icon_addplus.gif');
			$this->UserInterface->Toolbar->AddButton($this->Language->Get('help'), 'icon_help.gif', SWIFT_Help::RetrieveHelpLink('parserlooprule'),
					SWIFT_UserInterfaceToolbar::LINK_NEWWINDOW);
		}


		/*
		 * ###############################################
		 * BEGIN GENERAL TAB
		 * ###############################################
		 */
		$_GeneralTabObject = $this->UserInterface->AddTab($this->Language->Get('tabgeneral'), 'icon_form.gif', 'general', true);

		$_GeneralTabObject->Text('title', $this->Language->Get('thresholdruletitle'), $this->Language->Get('desc_thresholdruletitle'),
				$_loopRuleTitle);
		$_GeneralTabObject->Number('rulelength', $this->Language->Get('pr_newloopcontrolwatchlength_title'),
				$this->Language->Get('pr_newloopcontrolwatchlength_desc'), $_loopRuleLength);
		$_GeneralTabObject->Number('maxhits', $this->Language->Get('pr_newloopcontrolmaxcontacts_title'),
				$this->Language->Get('pr_newloopcontrolmaxcontacts_desc'), $_loopRuleMaxHits);
		$_GeneralTabObject->Number('restoreafter', $this->Language->Get('pr_newloopcontrolrestoreafter_title'),
				$this->Language->Get('pr_newloopcontrolrestoreafter_desc'), $_loopRuleRestoreAfter);

		/*
		 * ###############################################
		 * END GENERAL TAB
		 * ###############################################
		 */

		$this->UserInterface->End();

		return true;
	}

	/**
	 * Render the Parser Loop Rule Grid
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	public function RenderGrid()
	{
		if (!$this->GetIsClassLoaded())
		{
			throw new SWIFT_Exception(SWIFT_CLASSNOTLOADED);

			return false;
		}

		$this->Load->Library('UserInterface:UserInterfaceGrid', array('parserlooprulegrid'));

		if ($this->UserInterfaceGrid->GetMode() == SWIFT_UserInterfaceGrid::MODE_SEARCH)
		{
			$this->UserInterfaceGrid->SetSearchQuery('SELECT * FROM ' . TABLE_PREFIX . 'parserlooprules WHERE (' .
					$this->UserInterfaceGrid->BuildSQLSearch('title') . ')', 'SELECT COUNT(*) AS totalitems FROM ' . TABLE_PREFIX .
					'parserlooprules WHERE (' . $this->UserInterfaceGrid->BuildSQLSearch('title') . ')');
		}

		$this->UserInterfaceGrid->SetQuery('SELECT * FROM ' . TABLE_PREFIX . 'parserlooprules', 'SELECT COUNT(*) AS totalitems FROM ' .
				TABLE_PREFIX . 'parserlooprules');

		$this->UserInterfaceGrid->AddField(new SWIFT_UserInterfaceGridField('parserloopruleid', 'parserloopruleid',
				SWIFT_UserInterfaceGridField::TYPE_ID));

		$this->UserInterfaceGrid->AddField(new SWIFT_UserInterfaceGridField('icon', '&nbsp;', SWIFT_UserInterfaceGridField::TYPE_CUSTOM, 16,
				SWIFT_UserInterfaceGridField::ALIGN_CENTER));
		$this->UserInterfaceGrid->AddField(new SWIFT_UserInterfaceGridField('title', $this->Language->Get('pr_threshhold_grid_title'),
				SWIFT_UserInterfaceGridField::TYPE_DB, 0, SWIFT_UserInterfaceGridField::ALIGN_LEFT));
		$this->UserInterfaceGrid->AddField(new SWIFT_UserInterfaceGridField('length', $this->Language->Get('pr_threshhold_grid_timeframe_title'),
				SWIFT_UserInterfaceGridField::TYPE_DB, 180, SWIFT_UserInterfaceGridField::ALIGN_LEFT));
		$this->UserInterfaceGrid->AddField(new SWIFT_UserInterfaceGridField('maxhits', $this->Language->Get('pr_threshhold_grid_maxhits_title'),
				SWIFT_UserInterfaceGridField::TYPE_DB, 200, SWIFT_UserInterfaceGridField::ALIGN_LEFT));
		$this->UserInterfaceGrid->AddField(new SWIFT_UserInterfaceGridField('restoreafter',
				$this->Language->Get('pr_threshhold_grid_restoreafter_title'), SWIFT_UserInterfaceGridField::TYPE_DB, 180,
				SWIFT_UserInterfaceGridField::ALIGN_LEFT, SWIFT_UserInterfaceGridField::SORT_ASC), true);

		$this->UserInterfaceGrid->SetRenderCallback(array('View_LoopRule', 'GridRender'));
		$this->UserInterfaceGrid->AddMassAction(new SWIFT_UserInterfaceGridMassAction($this->Language->Get('delete'), 'icon_delete.gif',
				array('Controller_LoopRule', 'DeleteList'), $this->Language->Get('actionconfirm')));

		$this->UserInterfaceGrid->SetNewLinkViewport(SWIFT::Get('basename') . '/Parser/LoopRule/Insert');

		$this->UserInterfaceGrid->Render();

		$this->UserInterfaceGrid->Display();

		return true;
	}

	/**
	 * The Grid Rendering Function
	 *
	 * @author Varun Shoor
	 * @param array $_fieldContainer The Field Record Value Container
	 * @return bool "true" on Success, "false" otherwise
	 */
	static public function GridRender($_fieldContainer)
	{
		$_SWIFT = SWIFT::GetInstance();

		$_fieldContainer['length'] = intval($_fieldContainer['length']) . ' '. $_SWIFT->Language->Get('seconds');

		$_fieldContainer['restoreafter'] = intval($_fieldContainer['restoreafter']) . ' '. $_SWIFT->Language->Get('seconds');

		$_fieldContainer['icon'] = '<img src="' . SWIFT::Get('themepath') . 'images/icon_loopcutter.gif" align="absmiddle" border="0" />';

		$_fieldContainer['title'] = IIF($_fieldContainer['ismaster'] == '1', '<em>') . '<a href="' . SWIFT::Get('basename') . '/Parser/LoopRule/Edit/' . intval($_fieldContainer['parserloopruleid']) . '" onclick="' .
				"javascript: return UICreateWindowExtended(event, '" . SWIFT::Get('basename') . "/Parser/LoopRule/Edit/" . intval($_fieldContainer['parserloopruleid']) .
				"', 'editloopcutterrule', '" . $_SWIFT->Language->Get('wineditloopcutterrule') . "', '". $_SWIFT->Language->Get('loadingwindow') .
				"', 680, 500, true, this);" . '" title="' . $_SWIFT->Language->Get('edit') . '">' . htmlspecialchars($_fieldContainer['title']) .
				'</a>' . IIF($_fieldContainer['ismaster'] == '1', '</em>');

		return $_fieldContainer;
	}
}
?>