<?php

/**
 * ###############################################
 *
 * SWIFT Framework
 * _______________________________________________
 *
 * @author		Varun Shoor
 *
 * @package		SWIFT
 * @copyright	Copyright (c) 2001-2012, Kayako
 * @license		http://www.kayako.com/license
 * @link		http://www.kayako.com
 *
 * ###############################################
 */

/**
 * The OnSite Manager Class
 *
 * @author Varun Shoor
 */
class SWIFT_OnSiteManager extends SWIFT_Library
{
	// Core Constants
	const OS_WINDOWS = 'windows';
	const OS_MAC = 'mac';

	const SWIFT_ONSITEDIRECTORY = 'onsite/bin';
	const SWIFT_ONSITEFILENAMEWIN = 'OnSiteInstaller.exe';
	const SWIFT_DEFAULTONSITEFILENAME = 'OnSite-%s%s-%s.exe';
	const SWIFT_ONSITEUSESSLMARKER    = '[s]';

	/**
	 * Constructor
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function __construct()
	{
		parent::__construct();

		return true;
	}

	/**
	 * Destructor
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function __destruct()
	{
		parent::__destruct();

		return true;
	}

	/**
	 * Check to see if its a valid OS for OnSite Server
	 *
	 * @author Varun Shoor
	 * @param constant $_operatingSystem The Operating System
	 * @return bool "true" on Success, "false" otherwise
	 */
	static public function IsValidOS($_operatingSystem)
	{
		if ($_operatingSystem == self::OS_WINDOWS) {
			return true;
		}

		return false;
	}

	/**
	 * Dispatch the OnSite Server
	 *
	 * @author Varun Shoor
	 * @param object $_SWIFT_OnSiteSessionObject The SWIFT_OnSiteSession Object Pointer
	 * @param constant $_operatingSystem The Operating System,
	 * @return bool "true" on Success, "false" otherwise
	 * @throws SWIFT_Exception If the Class is not Loaded or If Invalid Data is Provided
	 */
	public function DispatchOnSite(SWIFT_OnSiteSession $_SWIFT_OnSiteSessionObject, $_operatingSystem)
	{
		if (!$this->GetIsClassLoaded()) {
			throw new SWIFT_Exception(SWIFT_CLASSNOTLOADED);
			return false;
		} else if (!$_SWIFT_OnSiteSessionObject instanceof SWIFT_OnSiteSession || !$_SWIFT_OnSiteSessionObject->GetIsClassLoaded() || !self::IsValidOS($_operatingSystem)) {
			throw new SWIFT_Exception(SWIFT_INVALIDDATA);
			return false;
		}

		$_onSiteFileName = '';
		if ($_operatingSystem == self::OS_WINDOWS) {
			$_onSiteFileName = SWIFT_BASEDIRECTORY . '/' . self::SWIFT_ONSITEDIRECTORY . '/' . self::SWIFT_ONSITEFILENAMEWIN;
		} else {
			throw new SWIFT_OnSite_Exception('Unsupported OS: ' . $_operatingSystem);
		}

		if (!file_exists($_onSiteFileName))
		{
			// No sense in continuing.
			throw new SWIFT_OnSite_Exception('OnSite one-click binary can\'t be located at "' . $_onSiteFileName . '"');
		}

		$_onSiteSessionCode = $_SWIFT_OnSiteSessionObject->GetProperty('sessioncode');
	    $_swiftPath 		= StripTrailingSlash(SWIFT::Get('swiftpath'));
	    $_sslPrefix 		= preg_match("@https://@i", $_swiftPath) ? self::SWIFT_ONSITEUSESSLMARKER : '';

		// http://wiki.kayako.com/display/INTDEV/Code-based+OnSite+protocol#Code-basedOnSiteprotocol-Encodedfilenames
		$_swiftURL = str_replace
					(
						array('/',   "\\",  '*',   '|',   ':',   '?',   '<',   '>',   '"',   '-',   '.'),
						array('_2F', '_5C', '_2A', '_7C', '_3A', '_3F', '_3C', '_3E', '_22', '_2D', '_2E'),
						str_replace(array('http://', 'https://'), '', $_swiftPath)
					);

		$_onSiteFileNameDispatch = sprintf(self::SWIFT_DEFAULTONSITEFILENAME, $_sslPrefix, $_swiftURL, $_onSiteSessionCode);

		if (isset($_SERVER['HTTP_USER_AGENT']) && preg_match('/MSIE/', $_SERVER['HTTP_USER_AGENT'])) {
			// IE Bug in download name workaround
			@ini_set('zlib.output_compression', 'Off');
		}

		@ini_set('zlib.output_compression', 0);
		@ini_set('implicit_flush', 1);

		// These headers are from http://php.net/manual/en/function.readfile.php
		header('Content-Description: File Transfer');
		header('Content-Type: application/octet-stream');
		header("Content-Disposition: attachment; filename=\"" . $_onSiteFileNameDispatch . "\"");
		header("Content-Transfer-Encoding: binary");
		header("Expires: 0");
		header("Cache-Control: must-revalidate");
		header("Content-Length: " . filesize($_onSiteFileName));
		@ob_clean();
		flush();

		@readfile($_onSiteFileName);

		return true;
	}

	/*
	  public function DispatchOnSite(SWIFT_OnSiteSession $_SWIFT_OnSiteSessionObject, $_operatingSystem)
	  {
	  if (!$this->GetIsClassLoaded())
	  {
	  throw new SWIFT_Exception(SWIFT_CLASSNOTLOADED);
	  return false;
	  } else if (!$_SWIFT_OnSiteSessionObject instanceof SWIFT_OnSiteSession || !$_SWIFT_OnSiteSessionObject->GetIsClassLoaded() || !self::IsValidOS($_operatingSystem)) {
	  throw new SWIFT_Exception(SWIFT_INVALIDDATA);
	  return false;
	  }

	  $_notificationURL = SWIFT::Get('swiftpath') . 'visitor/index.php?/LiveChat/OnSite/Notification/' . $_SWIFT_OnSiteSessionObject->GetOnSiteSessionID() . '/' . $_SWIFT_OnSiteSessionObject->GetProperty('sessionhash');

	  $_configID = $_SWIFT_OnSiteSessionObject->GetProperty('configid');

	  $_xmppHost = $this->Settings->Get('ons_xmpphost_' . $_configID);
	  $_xmppPort = $this->Settings->Get('ons_xmppport_' . $_configID);
	  $_stunHost = $this->Settings->Get('ons_stunhost_' . $_configID);
	  $_stunPort = $this->Settings->Get('ons_stunport_' . $_configID);
	  $_turnHost = $this->Settings->Get('ons_turnhost_' . $_configID);
	  $_turnPortUDP = $this->Settings->Get('ons_turnportudp_' . $_configID);
	  $_turnPortTCP = $this->Settings->Get('ons_turnporttcp_' . $_configID);

	  $_iniContents = $this->GetOnSiteINI($_SWIFT_OnSiteSessionObject->GetProperty('staffname'), $this->Settings->Get('general_companyname'), $_notificationURL,
	  $_SWIFT_OnSiteSessionObject->GetProperty('peerjid'), $_SWIFT_OnSiteSessionObject->GetProperty('localjid'),
	  $_xmppHost, $_xmppPort, $_stunHost, $_stunPort, $_turnHost, $_turnPortUDP, $_turnPortTCP);

	  $_onSiteFileName = '';
	  if ($_operatingSystem == self::OS_WINDOWS) {
	  $_onSiteFileName = SWIFT_BASEDIRECTORY . '/' . self::SWIFT_ONSITEDIRECTORY . '/' . self::SWIFT_ONSITEFILENAMEWIN;
	  } else {
	  throw new SWIFT_OnSite_Exception('Unsupported OS: ' . $_operatingSystem);
	  }

	  $_filePointer = fopen($_onSiteFileName, 'rb');
	  $_onSiteViewerContents = '';
	  if ($_filePointer)
	  {
	  while (!feof($_filePointer)) {
	  $_onSiteViewerContents .= fgets($_filePointer, 4096);
	  }

	  fclose($_filePointer);
	  }
	  else
	  {
	  throw new SWIFT_OnSite_Exception('Couldn\'t open \'' . $_onSiteFileName . '\' for reading!');
	  }

	  $_onSiteViewerContents .= "\0" . $_iniContents . sprintf("%04d", strlen($_iniContents));

	  if(isset($_SERVER['HTTP_USER_AGENT']) && preg_match('/MSIE/', $_SERVER['HTTP_USER_AGENT'])) {
	  // IE Bug in download name workaround
	  @ini_set('zlib.output_compression','Off');
	  }

	  @ini_set('zlib.output_compression', 0);
	  @ini_set('implicit_flush', 1);

	  header('Content-Type: application/octet-stream');
	  header("Content-Disposition: inline; filename=\"" . self::SWIFT_DEFAULTONSITEFILENAME . "\"");

	  header("Content-Transfer-Encoding: binary");

	  echo $_onSiteViewerContents;

	  return true;
	  }
	 */

	/**
	 * Format the OnSite INI file contents
	 *
	 * @author Ryan Lederman
	 * @return Boolean success value
	 *
	 * @param string $_staffName Name of the staff member initiating the OnSite session (UTF-8).
	 * @param string $_companyName Name of this organization (UTF-8).
	 * @param string $_notificationURL The URL to open when the one-click installer launches, to notify the server.
	 * @param string $_peerJID The JID that the staff PC is registering with the XMPP server.
	 * @param string $_localJID The JID that the customer PC should register with the XMPP server.
	 * @param string $_xmppHost The FQDN or IP address of the XMPP server.
	 * @param int $_xmppPort The port number of the XMPP server.
	 * @param string $_stunHost The FQDN or IP address of the STUN server.
	 * @param int $_stunPort The port number of the STUN server.
	 * @param string $_turnHost The FQDN or IP address of the TURN server.
	 * @param int $_turnPortUDP The UDP port number of the TURN server.
	 * @param int $_turnPortTCP The TCP port number of the TURN server.
	 * @throws SWIFT_Exception If the class is not loaded
	 */
	public function GetOnSiteINI($_staffName, $_companyName, $_notificationURL, $_peerJID, $_localJID, $_xmppHost, $_xmppPort, $_stunHost, $_stunPort, $_turnHost, $_turnPortUDP, $_turnPortTCP)
	{
		if (!$this->GetIsClassLoaded())
			throw new SWIFT_Exception(SWIFT_CLASSNOTLOADED);

		$_fileContents = sprintf("[OnSite]\r\nStaffName=%s\r\nCompanyName=%s\r\nNotificationURL=%s\r\nPeerJID=%s\r\n" .
				"LocalJID=%s\r\nXmppHost=%s\r\nXmppPort=%s\r\nSTUNHost=%s\r\nSTUNPort=%s\r\nTURNHost=%s\r\n" .
				"TURNPortUDP=%s\r\nTURNPortTCP=%s", $_staffName, $_companyName, $_notificationURL, $_peerJID, $_localJID, $_xmppHost, intval($_xmppPort), $_stunHost, intval($_stunPort), $_turnHost, intval($_turnPortUDP), intval($_turnPortTCP));

		return $_fileContents;
	}

}

?>