<?php
/**
 * ###############################################
 *
 * SWIFT Framework
 * _______________________________________________
 *
 * @author		Varun Shoor
 *
 * @package		SWIFT
 * @copyright	Copyright (c) 2001-2012, Kayako
 * @license		http://www.kayako.com/license
 * @link		http://www.kayako.com
 *
 * ###############################################
 */

/**
 * The OnSite UI Handling Lib
 *
 * @author Varun Shoor
 */
class SWIFT_OnSiteUI extends SWIFT_Library
{
	const BROWSER_IE7 = 'ie7';
	const BROWSER_IE8 = 'ie8';
	const BROWSER_IE9 = 'ie9';
	const BROWSER_OPERA = 'opera';
	const BROWSER_FIREFOX = 'firefox';
	const BROWSER_SAFARI = 'safari';
	const BROWSER_CHROME = 'chrome';

	const PLATFORM_WINDOWS = 'windows';
	const PLATFORM_OTHER = 'other';

	protected $_platform = false;
	protected $_browserType = false;

	protected $_baseTiles = array(
		self::BROWSER_IE7 => 3,
		self::BROWSER_IE8 => 3,
		self::BROWSER_IE9 => 1,
		self::BROWSER_SAFARI => 2,
		self::BROWSER_OPERA => 2,
		self::BROWSER_FIREFOX => 3,
		self::BROWSER_CHROME => 3,
		);


	/**
	 * Constructor
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function __construct()
	{
		parent::__construct();

		$this->Load->Library('UserAgent:UserAgent');

		$this->Language->Load('onsite');

		return true;
	}

	/**
	 * Destructor
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function __destruct()
	{
		parent::__destruct();

		return true;
	}

	/**
	 * Check to see if its a valid platform
	 *
	 * @author Varun Shoor
	 * @param constant $_platform
	 * @return bool "true" on Success, "false" otherwise
	 */
	static public function IsValidPlatform($_platformType)
	{
		return ($_platformType == self::PLATFORM_WINDOWS);
	}

	/**
	 * Check to see if its a valid browser
	 *
	 * @author Varun Shoor
	 * @param constant $_browserType
	 * @return bool "true" on Success, "false" otherwise
	 */
	static public function IsValidBrowser($_browserType)
	{
		return ($_browserType == self::BROWSER_IE7 || $_browserType == self::BROWSER_IE8 || $_browserType == self::BROWSER_IE9 || $_browserType == self::BROWSER_FIREFOX
				|| $_browserType == self::BROWSER_CHROME || $_browserType == self::BROWSER_SAFARI || $_browserType == self::BROWSER_OPERA);
	}

	/**
	 * Retrieves the Installation instruction tiles
	 *
	 * @author Varun Shoor
	 * @return array
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	protected function LoadTiles()
	{
		if (!$this->GetIsClassLoaded()) {
			throw new SWIFT_Exception(__CLASS__ . ':  ' . SWIFT_CLASSNOTLOADED);

			return false;
		}

		$_tilesContainer = array();

		$_browserList = $this->UserAgent->GetBrowserList();

		$_hasTiles = false;
		$_browserType = false;

		if (isset($_browserList['chrome']) && $_browserList['chrome'] == '1') {
			$_browserType = self::BROWSER_CHROME;
		} else if (isset($_browserList['firefox']) && $_browserList['firefox'] == '1') {
			$_browserType = self::BROWSER_FIREFOX;
		} else if (isset($_browserList['safari']) && $_browserList['safari'] == '1') {
			$_browserType = self::BROWSER_SAFARI;
		} else if (isset($_browserList['opera']) && $_browserList['opera'] == '1') {
			$_browserType = self::BROWSER_OPERA;
		} else if (isset($_browserList['ie7']) && $_browserList['ie7'] == '1') {
			$_browserType = self::BROWSER_IE7;
		} else if (isset($_browserList['ie8']) && $_browserList['ie8'] == '1') {
			$_browserType = self::BROWSER_IE8;
		} else if (isset($_browserList['ie9']) && $_browserList['ie9'] == '1') {
			$_browserType = self::BROWSER_IE9;
		} else if (isset($_browserList['ie']) && $_browserList['ie'] == '1') {
			$_browserType = self::BROWSER_IE9;
		}

		if (!isset($this->_baseTiles[$_browserType]) || empty($this->_platform)) {
			$this->Template->Assign('_hasTiles', false);

			return false;
		}

		$this->_browserType = $_browserType;
		$_hasTiles = true;

		for ($index = 1; $index <= $this->_baseTiles[$_browserType]; $index++) {
			$_tilesContainer[$index] = array(SWIFT::Get('themepath' . APP_ONSITE) . 'images/onsite/' . $this->_platform . '/' . $_browserType . '/image' . $index . '.png', $this->Language->Get($_browserType . '_tile' . $index . '_desc'), $index);
		}

		$this->Template->Assign('_tilesContainer', $_tilesContainer);
		$this->Template->Assign('_hasTiles', $_hasTiles);

		return true;
	}

	/**
	 * Loads the Platform
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	protected function LoadPlatform()
	{
		if (!$this->GetIsClassLoaded()) {
			throw new SWIFT_Exception(__CLASS__ . ':  ' . SWIFT_CLASSNOTLOADED);

			return false;
		}

		$_platformSupported = false;

		$_osList = $this->UserAgent->GetOSList();
		if (isset($_osList['win']) && $_osList['win'] == '1') {
			$_platformSupported = true;
			$this->_platform = self::PLATFORM_WINDOWS;
		}

//		$_platformSupported = true;
//		$this->_platform = self::PLATFORM_WINDOWS;

		$this->Template->Assign('_platformSupported', $_platformSupported);

		return true;
	}

	/**
	 * Display the OnSite UI
	 *
	 * @author Varun Shoor
	 * @param string $_sessionCode (OPTIONAL)
	 * @param string $_errorString (OPTIONAL)
	 * @return bool "true" on Success, "false" otherwise
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	public function Display($_sessionCode = false, $_errorString = '')
	{
		if (!$this->GetIsClassLoaded()) {
			throw new SWIFT_Exception(__CLASS__ . ':  ' . SWIFT_CLASSNOTLOADED);

			return false;
		}

		if (!is_numeric($_sessionCode)) {
			$_sessionCode = false;
		}

		$this->Template->Assign('_errorString', htmlspecialchars($_errorString));

		if (!empty($_sessionCode)) {
			$_linkURL = SWIFT::Get('swiftpath') . 'onsite/index.php?/OnSite/Session/Download/' . $_sessionCode;

			$this->Template->Assign('_sessionCode', $_sessionCode);
			$this->Template->Assign('_sessionDownloadLink', $_linkURL);
			$this->Template->Assign('_hasSessionCode', true);
			$this->Template->Assign('_onsiteIntroSession', sprintf($this->Language->Get('onsite_introsession'), $_linkURL));
		} else {
			$this->Template->Assign('_hasSessionCode', false);
		}

		$this->LoadPlatform();

		$this->LoadTiles();

		$this->Template->Render('onsitelanding');

		return true;
	}
}
?>